# Generated by roxygen2: do not edit by hand

S3method(plot,History)
S3method(print,History)
S3method(summary,History)
export(BatchContextualEpsilonGreedyPolicy)
export(BatchContextualLinTSPolicy)
export(BatchLinUCBDisjointPolicyEpsilon)
export(ContextualLinearBandit)
export(LinUCBDisjointPolicyEpsilon)
export(cram_bandit)
export(cram_bandit_est)
export(cram_bandit_sim)
export(cram_bandit_var)
export(cram_estimator)
export(cram_expected_loss)
export(cram_learning)
export(cram_ml)
export(cram_policy)
export(cram_policy_value_estimator)
export(cram_simulation)
export(cram_var_expected_loss)
export(cram_variance_estimator)
export(cram_variance_estimator_policy_value)
export(fit_model)
export(fit_model_ml)
export(get_betas)
export(ml_learning)
export(model_predict)
export(model_predict_ml)
export(set_model)
export(test_baseline_policy)
export(test_batch)
export(validate_params)
export(validate_params_fnn)
import(data.table)
import(glmnet)
import(keras)
import(rjson)
importFrom(DT,datatable)
importFrom(R.devices,devOptions)
importFrom(R6,R6Class)
importFrom(caret,train)
importFrom(caret,trainControl)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,fwrite)
importFrom(data.table,rbindlist)
importFrom(data.table,set)
importFrom(data.table,setcolorder)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(data.table,setorder)
importFrom(data.table,tstrsplit)
importFrom(data.table,uniqueN)
importFrom(doParallel,registerDoParallel)
importFrom(doParallel,stopImplicitCluster)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_modify)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(foreach,"%dopar%")
importFrom(foreach,`%do%`)
importFrom(foreach,`%dopar%`)
importFrom(foreach,foreach)
importFrom(glmnet,cv.glmnet)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.off)
importFrom(grDevices,graphics.off)
importFrom(grf,causal_forest)
importFrom(iterators,icount)
importFrom(itertools,isplitVector)
importFrom(keras,compile)
importFrom(keras,fit)
importFrom(keras,keras_model_sequential)
importFrom(keras,layer_dense)
importFrom(magrittr,"%>%")
importFrom(parallel,clusterExport)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(purrr,map2_dbl)
importFrom(rjson,toJSON)
importFrom(stats,D)
importFrom(stats,dgamma)
importFrom(stats,dnorm)
importFrom(stats,glm)
importFrom(stats,integrate)
importFrom(stats,pgamma)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qgamma)
importFrom(stats,qnorm)
importFrom(stats,rbeta)
importFrom(stats,rbinom)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,var)
